@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2016, University of Amsterdam
+    Copyright (c)  1985-2017, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -37,6 +37,7 @@
           [ '$initialise'/0,            % start Prolog
             '$toplevel'/0,              % Prolog top-level (re-entrant)
             '$compile'/0,               % `-c' toplevel
+            initialize/0,               % Run program initialization
             version/0,                  % Write initial banner
             version/1,                  % Add message to the banner
             prolog/0,                   % user toplevel predicate
@@ -222,6 +223,24 @@ prolog:message(initialize_now(Goal, Use)) -->
     '$run_initialization'(_, []),
     '$thread_init'.
 
+%!  initialize
+%
+%   Run goals registered with `:-  initialization(Goal, program).`. Stop
+%   with an exception if a goal fails or raises an exception.
+
+initialize :-
+    forall('$init_goal'(when(program), Goal, Ctx),
+           run_initialize(Goal, Ctx)).
+
+run_initialize(Goal, Ctx) :-
+    (   catch(Goal, E, true),
+        (   var(E)
+        ->  true
+        ;   throw(error(initialization_error(E, Goal, Ctx), _))
+        )
+    ;   throw(error(initialization_error(failed, Goal, Ctx), _))
+    ).
+
 
                  /*******************************
                  *     THREAD INITIALIZATION    *
@@ -321,8 +340,8 @@ path_sep -->
 
 %!  argv_files(-Files) is det.
 %
-%   Updated the prolog flag =argv=, extracting the leading directory
-%   and files.
+%   Update the Prolog flag =argv=, extracting  the leading directory and
+%   files.
 
 argv_files(Files) :-
     current_prolog_flag(argv, Argv),
@@ -477,6 +496,7 @@ initialise_error(E) :-
 
 initialise_prolog :-
     '$clean_history',
+    set_toplevel,
     associated_files(Files),
     '$set_file_search_paths',
     init_debug_flags,
@@ -487,11 +507,89 @@ initialise_prolog :-
     '$cmd_option_val'(init_file, OsFile),
     prolog_to_os_filename(File, OsFile),
     '$load_init_file'(File),
+    catch(setup_colors, E, print_message(warning, E)),
     '$load_script_file',
     load_associated_files(Files),
-    '$cmd_option_val'(goal, GoalAtom),
-    term_to_atom(Goal, GoalAtom),
-    ignore(user:Goal).
+    '$cmd_option_val'(goals, Goals),
+    (   Goals == [],
+        \+ '$init_goal'(when(_), _, _)
+    ->  version                                 % default interactive run
+    ;   run_init_goals(Goals),
+        (   load_only
+        ->  version
+        ;   run_program_init,
+            run_main_init
+        )
+    ).
+
+set_toplevel :-
+    '$cmd_option_val'(toplevel, TopLevelAtom),
+    catch(term_to_atom(TopLevel, TopLevelAtom), E,
+          (print_message(error, E),
+           halt(1))),
+    create_prolog_flag(toplevel_goal, TopLevel, [type(term)]).
+
+load_only :-
+    current_prolog_flag(os_argv, OSArgv),
+    memberchk('-l', OSArgv),
+    current_prolog_flag(argv, Argv),
+    \+ memberchk('-l', Argv).
+
+%!  run_init_goals(+Goals) is det.
+%
+%   Run registered initialization goals  on  order.   If  a  goal fails,
+%   execution is halted.
+
+run_init_goals([]).
+run_init_goals([H|T]) :-
+    run_init_goal(H),
+    run_init_goals(T).
+
+run_init_goal(Text) :-
+    catch(term_to_atom(Goal, Text), E,
+          (   print_message(error, init_goal_syntax(E, Text)),
+              halt(2)
+          )),
+    run_init_goal(Goal, Text).
+
+%!  run_program_init is det.
+%
+%   Run goals registered using
+
+run_program_init :-
+    forall('$init_goal'(when(program), Goal, Ctx),
+           run_init_goal(Goal, @(Goal,Ctx))).
+
+run_main_init :-
+    findall(Goal-Ctx, '$init_goal'(when(main), Goal, Ctx), Pairs),
+    '$last'(Pairs, Goal-Ctx),
+    !,
+    (   current_prolog_flag(toplevel_goal, default)
+    ->  set_prolog_flag(toplevel_goal, halt)
+    ;   true
+    ),
+    run_init_goal(Goal, @(Goal,Ctx)).
+run_main_init.
+
+run_init_goal(Goal, Ctx) :-
+    (   catch(user:Goal, E, true)
+    ->  (   var(E)
+        ->  true
+        ;   print_message(error, init_goal_failed(E, Ctx)),
+            halt(2)
+        )
+    ;   (   current_prolog_flag(verbose, silent)
+        ->  Level = silent
+        ;   Level = error
+        ),
+        print_message(Level, init_goal_failed(failed, Ctx)),
+        halt(1)
+    ).
+
+%!  init_debug_flags is det.
+%
+%   Initialize the various Prolog flags that   control  the debugger and
+%   toplevel.
 
 init_debug_flags :-
     once(print_predicate(_, [print], PrintOptions)),
@@ -521,10 +619,10 @@ setup_backtrace :-
 %   Setup  interactive  usage  by  enabling    colored   output.
 
 setup_colors :-
-    (   stream_property(user_input, tty(true)),
+    (   \+ current_prolog_flag(color_term, false),
+        stream_property(user_input, tty(true)),
         stream_property(user_error, tty(true)),
         stream_property(user_output, tty(true)),
-        \+ current_prolog_flag(color_term, false),
         load_setup_file(user:library(ansi_term))
     ->  true
     ;   true
@@ -542,7 +640,8 @@ setup_history :-
     ->  prolog_history(enable)
     ;   true
     ),
-    set_default_history.
+    set_default_history,
+    '$load_history'.
 
 %!  setup_readline
 %
@@ -553,6 +652,7 @@ setup_readline :-
     ->  true
     ;   stream_property(user_input, tty(true)),
         current_prolog_flag(tty_control, true),
+        \+ getenv('TERM', dumb),
         (   current_prolog_flag(readline, ReadLine)
         ->  true
         ;   ReadLine = true
@@ -597,37 +697,36 @@ load_setup_file(File) :-
 
 %!  '$runtoplevel'
 %
-%   Actually run the toplevel. If there  is   a  syntax error in the
-%   goal there is no reason to   persue.  Something like that should
-%   happen to repetitive exceptions in the toplevel as well, but how
-%   do we distinguish between  interactive   usage  that  frequently
-%   raises and error and a program crashing in a loop?
+%   Actually run the toplevel. The values   `default`  and `prolog` both
+%   start the interactive toplevel, where `prolog` implies the user gave
+%   =|-t prolog|=.
 %
 %   @see prolog/0 is the default interactive toplevel
 
 '$runtoplevel' :-
-    '$cmd_option_val'(toplevel, TopLevelAtom),
-    catch(term_to_atom(TopLevel0, TopLevelAtom), E,
-          (print_message(error, E),
-           halt(1))),
+    current_prolog_flag(toplevel_goal, TopLevel0),
     toplevel_goal(TopLevel0, TopLevel),
     user:TopLevel.
 
 :- dynamic  setup_done/0.
 :- volatile setup_done/0.
 
+toplevel_goal(default, '$query_loop') :-
+    !,
+    setup_interactive.
 toplevel_goal(prolog, '$query_loop') :-
     !,
-    (   setup_done
-    ->  true
-    ;   asserta(setup_done),
-        catch(setup_backtrace, E, print_message(warning, E)),
-        catch(setup_colors,    E, print_message(warning, E)),
-        catch(setup_readline,  E, print_message(warning, E)),
-        catch(setup_history,   E, print_message(warning, E))
-    ).
+    setup_interactive.
 toplevel_goal(Goal, Goal).
 
+setup_interactive :-
+    setup_done,
+    !.
+setup_interactive :-
+    asserta(setup_done),
+    catch(setup_backtrace, E, print_message(warning, E)),
+    catch(setup_readline,  E, print_message(warning, E)),
+    catch(setup_history,   E, print_message(warning, E)).
 
 %!  '$compile'
 %
@@ -817,7 +916,7 @@ set_default_history :-
     current_prolog_flag(history, _),
     !.
 set_default_history :-
-    (   (   current_prolog_flag(readline, true)
+    (   (   \+ current_prolog_flag(readline, false)
         ;   current_prolog_flag(emacs_inferior_process, true)
         )
     ->  create_prolog_flag(history, 0, [])
