@@ -399,8 +399,8 @@ accept_server3(Goal, Options) :-
     memberchk(queue(Queue), Options),
     tcp_accept(Socket, Client, Peer),
     debug(http(connection), 'New HTTP connection from ~p', [Peer]),
-    http_enough_workers(Queue, accept, Peer),
-    thread_send_message(Queue, tcp_client(Client, Goal, Peer)).
+    thread_send_message(Queue, tcp_client(Client, Goal, Peer)),
+    http_enough_workers(Queue, accept, Peer).
 
 accept_rethrow_error(http_stop).
 accept_rethrow_error('$aborted').
@@ -459,7 +459,9 @@ http_enough_workers(Queue, Why, Peer) :-
         catch(http:schedule_workers(_{port:Port,
                                       reason:Why,
                                       peer:Peer,
-                                      waiting:Size}),
+                                      waiting:Size,
+                                      queue:Queue
+                                     }),
               Error,
               print_message(error, Error))
     ->  true
@@ -485,6 +487,8 @@ enough(1, keep_alive).                  % I will be ready myself
 %     Identify the other end of the connection
 %     - waiting:Size
 %     Number of messages waiting in the queue.
+%     - queue:Queue
+%     Message queue used to dispatch accepted messages.
 %
 %   Note that, when called with `reason:accept`,   we  are called in
 %   the time critical main accept loop.   An  implementation of this
@@ -675,12 +679,12 @@ check_keep_alive_connection(In, TMO, Peer, In, Out) :-
 
 done_worker :-
     thread_self(Self),
+    thread_detach(Self),
     retract(queue_worker(Queue, Self)),
     thread_property(Self, status(Status)),
     !,
     (   catch(recreate_worker(Status, Queue), _, fail)
-    ->  thread_detach(Self),
-        print_message(informational,
+    ->  print_message(informational,
                       httpd_restarted_worker(Self))
     ;   done_status_message_level(Status, Level),
         print_message(Level,
