@@ -209,7 +209,9 @@ print_backtrace(int last)		/* 1..SAVE_TRACES */
 
 void
 bstore_print_backtrace_named(btrace *bt, const char *why)
-{ if ( bt )
+{ int done = 0;
+
+  if ( bt )
   { int me = bt->current-1;
 
     for(;;)
@@ -217,14 +219,15 @@ bstore_print_backtrace_named(btrace *bt, const char *why)
 	me += SAVE_TRACES;
       if ( bt->dumps[me].name && strcmp(bt->dumps[me].name, why) == 0 )
       { print_trace(bt, me);
-	return;
+	done++;
       }
       if ( --me == bt->current-1 )
 	break;
     }
   }
 
-  Sdprintf("No backtrace named %s\n", why);
+  if ( !done )
+    Sdprintf("No backtrace named %s\n", why);
 }
 
 #endif /*HAVE_LIBUNWIND*/
@@ -440,7 +443,9 @@ print_backtrace(int last)		/* 1..SAVE_TRACES */
 
 void
 bstore_print_backtrace_named(btrace *bt, const char *why)
-{ if ( bt )
+{ int done = 0;
+
+  if ( bt )
   { int me = bt->current-1;
 
     for(;;)
@@ -448,14 +453,15 @@ bstore_print_backtrace_named(btrace *bt, const char *why)
 	me += SAVE_TRACES;
       if ( bt->why[me] && strcmp(bt->why[me], why) == 0 )
       { print_trace(bt, me);
-	return;
+	done++;
       }
       if ( --me == bt->current-1 )
 	break;
     }
   }
 
-  Sdprintf("No backtrace named %s\n", why);
+  if ( !done )
+    Sdprintf("No backtrace named %s\n", why);
 }
 
 
@@ -820,7 +826,9 @@ print_backtrace(int last)		/* 1..SAVE_TRACES */
 
 void
 bstore_print_backtrace_named(btrace *bt, const char *why)
-{ if ( bt )
+{ int done = 0;
+
+  if ( bt )
   { int me = bt->current-1;
 
     for(;;)
@@ -828,14 +836,15 @@ bstore_print_backtrace_named(btrace *bt, const char *why)
 	me += SAVE_TRACES;
       if ( bt->dumps[me].name && strcmp(bt->dumps[me].name, why) == 0 )
       { print_trace(bt, me);
-	return;
+	done++;
       }
       if ( --me == bt->current-1 )
 	break;
     }
   }
 
-  Sdprintf("No backtrace named %s\n", why);
+  if ( !done )
+    Sdprintf("No backtrace named %s\n", why);
 }
 
 static LONG WINAPI crashHandler(PEXCEPTION_POINTERS pExceptionInfo)
@@ -929,6 +938,45 @@ sigCrashHandler(int sig)
 
 #endif /*BTRACE_DONE*/
 
+		 /*******************************
+		 *	PROLOG CONNECTION	*
+		 *******************************/
+
+#if defined(O_DEBUG) && defined(BTRACE_DONE)
+#define BTRACE_PREDS 1
+
+static
+PRED_IMPL("c_backtrace_clear", 0, c_backtrace_clear, 0)
+{ PRED_LD
+
+  if ( LD->btrace_store )
+  { btrace_destroy(LD->btrace_store);
+    LD->btrace_store = NULL;
+  }
+
+  return TRUE;
+}
+
+static
+PRED_IMPL("c_backtrace_print", 1, c_backtrace_print, 0)
+{ char *s;
+
+  if ( PL_get_chars(A1, &s, CVT_ATOM|CVT_STRING|CVT_EXCEPTION) )
+  { print_backtrace_named(s);
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+#endif
+
+BeginPredDefs(cbtrace)
+#ifdef BTRACE_PREDS
+  PRED_DEF("c_backtrace_clear", 0, c_backtrace_clear, 0)
+  PRED_DEF("c_backtrace_print", 1, c_backtrace_print, 0)
+#endif
+EndPredDefs
 
 	         /*******************************
 		 *   FALLBACK IMPLEMENTATION	*
