@@ -2430,8 +2430,10 @@ delayEvent(pl_event_type ev, va_list args)
     dev.type = ev;
 
     switch(ev)
-    { case PLEV_BREAK:
+    { case PLEV_BREAK_EXISTS:
+      case PLEV_BREAK:
       case PLEV_NOBREAK:
+      case PLEV_GCNOBREAK:
 	dev.value.pc.clause = va_arg(args, Clause);
 	dev.value.pc.offset = va_arg(args, int);
 	break;
@@ -2486,8 +2488,10 @@ sendDelayedEvents(int noerror)
     for(; count-- > 0; dev++)
     { if ( noerror )
       { switch(dev->type)
-	{ case PLEV_BREAK:
+	{ case PLEV_BREAK_EXISTS:
+	  case PLEV_BREAK:
 	  case PLEV_NOBREAK:
+	  case PLEV_GCNOBREAK:
 	    noerror = callEventHook(dev->type,
 				    dev->value.pc.clause, dev->value.pc.offset);
 	    sent++;
@@ -2555,9 +2559,10 @@ PL_call_event_hook_va(pl_event_type ev, va_list args)
     }
     case PLEV_ERASED_CLAUSE:
     { Clause cl = va_arg(args, Clause);		/* object erased */
-      term_t dbref = PL_new_term_ref();
+      term_t dbref;
 
-      rc = (  PL_unify_clref(dbref, cl) &&
+      rc = (  (dbref = PL_new_term_ref()) &&
+	      PL_unify_clref(dbref, cl) &&
 	      PL_unify_term(arg,
 			    PL_FUNCTOR, FUNCTOR_erased1,
 			      PL_TERM, dbref)
@@ -2566,9 +2571,10 @@ PL_call_event_hook_va(pl_event_type ev, va_list args)
     }
     case PLEV_ERASED_RECORD:
     { RecordRef r = va_arg(args, RecordRef);	/* object erased */
-      term_t dbref = PL_new_term_ref();
+      term_t dbref;
 
-      rc = (  PL_unify_recref(dbref, r) &&
+      rc = (  (dbref = PL_new_term_ref()) &&
+	      PL_unify_recref(dbref, r) &&
 	      PL_unify_term(arg,
 			    PL_FUNCTOR, FUNCTOR_erased1,
 			      PL_TERM, dbref)
@@ -2592,19 +2598,23 @@ PL_call_event_hook_va(pl_event_type ev, va_list args)
       break;
     }
     case PLEV_BREAK:
+    case PLEV_BREAK_EXISTS:
     case PLEV_NOBREAK:
+    case PLEV_GCNOBREAK:
     { Clause clause = va_arg(args, Clause);
       int offset = va_arg(args, int);
-      term_t cref = PL_new_term_ref();
+      term_t cref;
 
-
-      rc = ( PL_unify_clref(cref, clause) &&
+      rc = ( (cref = PL_new_term_ref()) &&
+	     PL_unify_clref(cref, clause) &&
 	     PL_unify_term(arg,
 			   PL_FUNCTOR, FUNCTOR_break3,
 			     PL_TERM, cref,
 			     PL_INT, offset,
-			     PL_ATOM, ev == PLEV_BREAK ? ATOM_true
-						       : ATOM_false)
+			     PL_ATOM, ev == PLEV_BREAK     ? ATOM_true :
+				      ev == PLEV_NOBREAK   ? ATOM_false :
+				      ev == PLEV_GCNOBREAK ? ATOM_gc :
+							     ATOM_exist)
 	   );
       break;
     }
@@ -2612,10 +2622,12 @@ PL_call_event_hook_va(pl_event_type ev, va_list args)
     { LocalFrame fr = va_arg(args, LocalFrame);
       term_t ref = PL_new_term_ref();
 
-      PL_put_frame(ref, fr);
-      rc = PL_unify_term(arg,
-			 PL_FUNCTOR, FUNCTOR_frame_finished1,
-			   PL_TERM, ref);
+      rc = ( (ref = PL_new_term_ref()) &&
+	     (PL_put_frame(ref, fr),TRUE) &&
+	     PL_unify_term(arg,
+			   PL_FUNCTOR, FUNCTOR_frame_finished1,
+			     PL_TERM, ref)
+	   );
       break;
     }
 #ifdef O_PLMT
